#!/usr/bin/python3

from configparser import ConfigParser
import datetime
import errno
import getopt
import json
import logging
import math
import os
import pwd
import sqlite3
import sys
import threading
from threading import current_thread
import time
import itertools
from functools import reduce
import pwd
import grp

#config file full path
config_path = '/usr/local/safesquid/ui_root/report/config.ini';

# set config initialization
config = ConfigParser()

# parse existing file
config.read(config_path)
# read values from a section
log_path = config.get('log_info', 'log_path')
log_file = config.get('log_info', 'log_file')
report_dir = config.get('report_path', 'report_dir')
user = config.get('permission', 'user')
group = config.get('permission', 'group')
QUERY_METHOD="ARG"

# logs to be written to file
logger = logging.getLogger(__name__)
logger.setLevel(logging.INFO)


uid = pwd.getpwnam("ssquid").pw_uid
gid = grp.getgrnam("root").gr_gid

# create a file handler
log_file_path = log_path + log_file

handler = logging.FileHandler(log_file_path)
handler.setLevel(logging.INFO)
pattern = "%Y/%m/%d"

# create a logging format
formatter = logging.Formatter('%(asctime)s - %(name)s - [ %(funcName)s():%(lineno)3s ] - %(levelname)s - %(message)s')
handler.setFormatter(formatter)

# add the handlers to the logger
logger.addHandler(handler)

if os.path.exists(log_file_path):
	os.chown(log_file_path, uid, gid)
	logger.debug('log_file_check: present')
else:
   open(log_file_path, 'w')
   os.chown(log_file_path, uid, gid)

print ('Content-type: application/json; charset=utf-8\r\n')
   
if QUERY_METHOD == "CMD":
	try:
		opts, args = getopt.getopt(sys.argv[1:], "hs:e:d:")
	except getopt.GetoptError as err:
		print (str(err))    
		sys.exit(2)
	
	if (len(opts) == 0):
		logger.error('No arg passed')
		usage()
		sys.exit(2)
	else:
		logger.info('total arg passed: %s', len(opts))
	
		for opt, arg in opts:
			if opt == '-h':
				usage()
				sys.exit(2)
			elif opt in ("-s"):
				START_TIME = arg            
			elif opt in ("-e"):
				END_TIME = arg
else:
	START_TIME = os.environ['START_TIME']
	END_TIME = os.environ['END_TIME']
	TYPE = os.environ['type']
		
def date_diff(fromdata,todate):	
	date_format = "%Y/%m/%d"
	a = datetime.datetime.strptime(fromdata, date_format)
	b = datetime.datetime.strptime(todate, date_format)
	delta = b - a
	return delta.days


def regroup(values,key_match):
    groups = dict()
    for d in values:
        key = (d[key_match])
        if key in groups:
            group = groups[key]
            group["rq"] += d["rq"]
            group["bw"] += d["bw"]
        else:
            groups[key] = d.copy()
    return list(groups.values())	
	
def regroup_2(values,key_match1,key_match2):
    groups = dict()
    for d in values:
        key = (d[key_match1],d[key_match2])
        if key in groups:
            group = groups[key]
            group["rq"] += d["rq"]
            group["bw"] += d["bw"]
        else:
            groups[key] = d.copy()
    return list(groups.values())	
	
if START_TIME == '-':
	START_TIME = END_TIME
	
#check date range	
days = date_diff(START_TIME,END_TIME) + 1	
t_ = END_TIME

UNIQ_USER = []
USER=[]
USER_= {}
SUMMARY = []
SUMMARY_ = {}
allowed_sites = []
ALLOWED_SITES = []	
denied_sites = []
DENIED_SITES = []

for x in range(days):	
	lastHourDateTime = datetime.datetime.strptime(t_, pattern) - datetime.timedelta(days=x)
	that_date = lastHourDateTime.strftime("%Y/%m/%d")
	
	that_data_dir = that_date.replace('.','/')
	fetch_dir = report_dir + "/" + that_data_dir
	
	if TYPE == "allowed_site":			
		fetch_allow = fetch_dir + "/allowed.json"	
		try:
			with open(fetch_allow, 'r') as myfile:
				allow = myfile.read()			
				allowed_sites.append(allow)
		except FileNotFoundError as er:        
			logger.error('%s', er)
	
	elif TYPE == "denied_sites":			
		fetch_denied = fetch_dir + "/denied.json"	
		try:
			with open(fetch_denied, 'r') as myfile:
				denied = myfile.read()			
				denied_sites.append(denied)
		except FileNotFoundError as er:        
			logger.error('%s', er)
			
	else :
		if TYPE == "summary":	
			fetch_summary = fetch_dir + "/summary.json"	
			try:
				with open(fetch_summary, 'r') as myfile:
					summary = myfile.read()			
					SUMMARY.append(summary)
			except FileNotFoundError as er:        
				logger.error('%s', er)
		
		
		fetch_report = fetch_dir + "/report.json"	
		try:
			with open(fetch_report, 'r') as myfile:
				report = myfile.read()	
				USER.append(report)
				UNIQ_USER.append(list(json.loads(report).keys()))			
		except FileNotFoundError as er:        
			logger.error('%s', er)

ALL_USER = list(itertools.chain.from_iterable(UNIQ_USER))
USER_LIST = sorted(set(ALL_USER))

if 	TYPE == "summary":	
	bw = 0		
	rq = 0
	bl = 0
	al = 0
	ha_ = []
	
	for x in range(len(list(SUMMARY))):		
		
		try:
			bw += json.loads(SUMMARY[x])['bw']
		except TypeError:
			bw += 0
			
		rq += json.loads(SUMMARY[x])['rq']
		bl += json.loads(SUMMARY[x])['bl']
		al += json.loads(SUMMARY[x])['al']
		
		ha_content = json.loads(SUMMARY[x])['ha']	
		for m in range(len(ha_content)):		
			ha_.append(ha_content[m])	
	
	SUMMARY_['bw'] = bw
	SUMMARY_['rq'] = rq
	SUMMARY_['bl'] = bl
	SUMMARY_['al'] = al
	
	del bw
	del rq
	del bl
	del al
	
	SUMMARY_['start'] = START_TIME
	SUMMARY_['end'] = END_TIME
	SUMMARY_['us'] = len(USER_LIST)
	SUMMARY_['ha'] = regroup(ha_,"interval")
	
elif TYPE == "user":
	dictA = {}
	USER_['users'] = USER_LIST
	USER_['content'] = {}
	
	for user in USER_LIST:
		dictA[user] = []
		USER_['content'][user] = {}
		
	#print (USER_)
	for x in range(len(list(USER))):
		dictB = json.loads(USER[x])
		
		for key in dictB:
			dictA[key].append(dictB[key])
		
	for user in USER_LIST:
		dict1 = dictA[user]
		
		tm = 0
		bw = 0
		rq = 0
		al = 0		
		dw = 0
		bl = 0
		ul = 0
		ha_ = []
		bl_l = []
		al_l = []
		
		for v in range(len(dict1)):
			tm += dict1[v]['tm']
			bw += dict1[v]['bw']
			rq += dict1[v]['rq']
			al += dict1[v]['al']
			dw += dict1[v]['dw']
			
			try:
				bl += dict1[v]['bl']
			except KeyError:			
				bl += 0
				
			ul += dict1[v]['ul']
			
			ha_content = dict1[v]['hour']
			for m1 in range(len(ha_content)):		
				ha_.append(ha_content[m1])
				
			bl_l_content = dict1[v]['blocked']
			for m2 in range(len(bl_l_content)):		
				bl_l.append(bl_l_content[m2])
			
			al_l_content = dict1[v]['allowed']
			for m3 in range(len(al_l_content)):		
				al_l.append(al_l_content[m3])
			
			
		USER_['content'][user]['tm'] = tm				
		USER_['content'][user]['bw'] = bw
		USER_['content'][user]['rq'] = rq
		USER_['content'][user]['al'] = al				
		USER_['content'][user]['dw'] = dw
		USER_['content'][user]['bl'] = bl				
		USER_['content'][user]['ul'] = ul
		USER_['content'][user]['ha'] = regroup(ha_,"int")
		USER_['content'][user]['allow'] = regroup(al_l,"al")
		USER_['content'][user]['blocked'] = regroup_2(bl_l,"bl","fn")

elif TYPE == "allowed_site":
	allowed_content = []
	
	for x_ in range(len(list(allowed_sites))):		
		current_content = json.loads(allowed_sites[x_])
		for x1 in range(len(list(current_content))):
			allowed_content.append(current_content[x1])
	
	ALLOWED_SITES = regroup(allowed_content,"url")	
	
elif TYPE == "denied_sites":
	denied_content = []
	
	for x_ in range(len(list(denied_sites))):		
		current_content = json.loads(denied_sites[x_])
		for x1 in range(len(list(current_content))):
			denied_content.append(current_content[x1])
	
	DENIED_SITES = regroup(denied_content,"url")		
		
if 	TYPE == "summary":		
	print(json.dumps(SUMMARY_))
elif TYPE == "user":
	print(json.dumps(USER_))
	logger.info('user in process')
elif TYPE == "allowed_site":
	print(json.dumps(ALLOWED_SITES))
elif TYPE == "denied_sites":
	print(json.dumps(DENIED_SITES))	